//////////////////////////////////////////////////////////////////////////////////////
//  
//  ColorSwatchPC.cp - Sample support file for FileMaker plug-in
//
//  Copyright 2003-2005 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky & HOnza Koudelka
//
////////////////////////////////////////////////////////////////////////////////
//
//  This example is intended for a FileMaker plug-in based on 24U FM Template,
//  a stationery from 24u Software.
//
//  The latest version of 24U FM Template is available for download from:
//
//  http://24uSoftware.com/FMTemplate  or  http://www.NewMillennium.com/
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source with references to
//  24U Software, we require that you make it clear in the source that
//  the code was descended from 24U Sample Code, but that you've made changes.
//
////////////////////////////////////////////////////////////////////////

//
//  This file contains Windows implementation of Color Swatch
//
////////////////////////////////////////////////////////////////////////////////

#include <Windows.h>
#include <commdlg.h>
#include "ColorSwatchPC.h"

static void CSDrawOuterRectangle(const LPDRAWITEMSTRUCT iDS)
{
    HGDIOBJ curPen;
    HPEN    lightPen, darkPen;
    RECT    r;
    
    lightPen   = CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNHIGHLIGHT));
    darkPen    = CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNSHADOW));
    
    if(iDS->itemState & ODS_SELECTED )
        curPen = SelectObject(iDS->hDC,darkPen);
    else
        curPen = SelectObject(iDS->hDC,lightPen);

    
    memcpy(&r,&iDS->rcItem,sizeof(RECT));   
    InflateRect(&r,-1,-1);
    MoveToEx(iDS->hDC,r.left,r.bottom,NULL);
    LineTo(iDS->hDC,r.left,r.top);
    LineTo(iDS->hDC,r.right-1,r.top);
    
    if(iDS->itemState & ODS_SELECTED )
        curPen = SelectObject(iDS->hDC,lightPen);
    else
        curPen = SelectObject(iDS->hDC,darkPen);

    LineTo(iDS->hDC,r.right-1,r.bottom-1);
    LineTo(iDS->hDC,r.left,r.bottom-1);
    
    SelectObject(iDS->hDC, curPen);
    
    DeleteObject(darkPen);
    DeleteObject(lightPen);
}

static void CSDrawInnerRectangle(const LPDRAWITEMSTRUCT iDS)
{
    HGDIOBJ curPen;
    HPEN    lightPen, darkPen;
    RECT    r;
    
    lightPen   = CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNHIGHLIGHT));
    darkPen    = CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNSHADOW));
    
    curPen = SelectObject(iDS->hDC,darkPen);
    
    memcpy(&r,&iDS->rcItem,sizeof(RECT));   
    InflateRect(&r,-3,-3);
    
    MoveToEx(iDS->hDC,r.left,r.bottom-1,NULL);
    LineTo(iDS->hDC,r.left,r.top);
    LineTo(iDS->hDC,r.right-1,r.top);
    
    curPen = SelectObject(iDS->hDC,lightPen);

    LineTo(iDS->hDC,r.right-1,r.bottom-1);
    LineTo(iDS->hDC,r.left,r.bottom-1);
    
    SelectObject(iDS->hDC, curPen);
    
    DeleteObject(darkPen);
    DeleteObject(lightPen);
}

static void CSDrawBackground(const LPDRAWITEMSTRUCT iDS)
{
    HGDIOBJ curPen, curBrush;
    HPEN    bgPen;
    HBRUSH  bgBrush;
    
    bgPen   = CreatePen(PS_SOLID,1,GetSysColor(COLOR_BTNFACE));
    bgBrush = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
    
    curPen = SelectObject(iDS->hDC,bgPen);
    curBrush = SelectObject(iDS->hDC,bgBrush);
    
    Rectangle(iDS->hDC,iDS->rcItem.left,iDS->rcItem.top,iDS->rcItem.right-1,iDS->rcItem.bottom-1);
    
    SelectObject(iDS->hDC, curPen);
    SelectObject(iDS->hDC, curBrush);
    
    DeleteObject(bgPen);
    DeleteObject(bgBrush);
}

static void CSDarkenColor(COLORREF& ioColor)
{
    unsigned short r = GetRValue(ioColor)*3>>2;
    unsigned short g = GetGValue(ioColor)*3>>2;
    unsigned short b = GetBValue(ioColor)*3>>2;
    
    ioColor = RGB(r,g,b);
}


static void CSDrawColor(const LPDRAWITEMSTRUCT iDS)
{
    HGDIOBJ     curPen, curBrush;
    HBRUSH      rectBrush;
    HPEN        rectPen;
    RECT        r;
    COLORREF    color = (COLORREF)GetWindowLong(iDS->hwndItem,GWL_USERDATA);


    if(iDS->itemState & ODS_SELECTED )
    {
        CSDarkenColor(color);
        rectBrush = CreateSolidBrush(color);
        rectPen   = CreatePen(PS_SOLID,1,color);
    }
    else
    {
        rectBrush = CreateSolidBrush(color);
        rectPen   = CreatePen(PS_SOLID,1,color);
    }
    
    curPen = SelectObject(iDS->hDC,rectPen);
    curBrush = SelectObject(iDS->hDC,rectBrush);
    
    memcpy(&r,&iDS->rcItem,sizeof(RECT));   
    
    InflateRect(&r,-4,-4);
    Rectangle(iDS->hDC, r.left, r.top, r.right, r.bottom);
    
    SelectObject(iDS->hDC, curPen);
    SelectObject(iDS->hDC, curBrush);
    
    DeleteObject(rectPen);
    DeleteObject(rectBrush);
}

static void CSDrawFocus(const LPDRAWITEMSTRUCT iDS)
{
    HGDIOBJ     curPen, curBrush;
    HBRUSH      rectBrush;
    HPEN        rectPen;
    RECT        r;
    COLORREF    color = 0;  

    rectPen   = CreatePen(PS_DOT,1,color);
    rectBrush   = (HBRUSH)GetStockObject(HOLLOW_BRUSH);
    
    curPen = SelectObject(iDS->hDC,rectPen);
    curBrush = SelectObject(iDS->hDC,rectBrush);
    
    memcpy(&r,&iDS->rcItem,sizeof(RECT));   
    
    InflateRect(&r,-5,-5);
    Rectangle(iDS->hDC, r.left, r.top, r.right, r.bottom);
    
    SelectObject(iDS->hDC, curPen);
    SelectObject(iDS->hDC, curBrush);
    
    DeleteObject(rectPen);
    DeleteObject(rectBrush);
}

#pragma mark -

void ColorSwatchClick(HWND iSwatch)
{
    BOOL            success;
    CHOOSECOLOR     cc;
    static COLORREF customColorBuffer[16];
        
    cc.lStructSize = sizeof(CHOOSECOLOR);
    cc.hwndOwner = iSwatch;
    cc.rgbResult = ColorSwatchGetColor(iSwatch);
    cc.lpCustColors = (LPDWORD)customColorBuffer;
    cc.Flags = CC_ANYCOLOR | CC_RGBINIT| CC_FULLOPEN;
    success = ChooseColor( &cc );
    
    if( success )
        ColorSwatchSetColor(iSwatch,cc.rgbResult);
}


void ColorSwatchDraw(const LPDRAWITEMSTRUCT iDS)
{
    CSDrawBackground(iDS);
    CSDrawOuterRectangle(iDS);
    CSDrawInnerRectangle(iDS);
    CSDrawColor(iDS);
    if(iDS->itemState & ODS_FOCUS)
      CSDrawFocus(iDS);
}

HWND ColorSwatchCreate(HWND iParent, const RECT* iRect, COLORREF iColor)
{
    HWND    result;
    
    result = CreateWindow(  "BUTTON","",
                            WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
                            iRect->left,iRect->top,iRect->right,iRect->bottom,
                            iParent,0,0,0);
    if( result )
    {
        ColorSwatchSetColor(result,iColor);
    }
    
    return result;
}

void ColorSwatchMakeFrom(HWND iSwatch, COLORREF iColor)
{
    DWORD style = GetWindowLong(iSwatch,GWL_STYLE);
    
    style |= BS_OWNERDRAW;
    
    SetWindowLong(iSwatch,GWL_STYLE,style);
    ColorSwatchSetColor(iSwatch,iColor);
}


COLORREF ColorSwatchGetColor(HWND iSwatch)
{
    return (COLORREF)GetWindowLong(iSwatch,GWL_USERDATA);
}

void ColorSwatchSetColor(HWND iSwatch, COLORREF iColor)
{
    SetWindowLong(iSwatch,GWL_USERDATA,(LONG)iColor);
    InvalidateRgn(iSwatch,NULL,FALSE);
}
